rails_env   = ENV['RAILS_ENV']  || "production"
rails_root  = ENV['RAILS_ROOT'] || File.dirname(__FILE__) + '/..'
user_home = ENV['HOME'] || File.dirname(__FILE__) + '/../..'
development_uid = ''
development_gid = ''

num_workers = rails_env == 'production' ? 5 : 2

num_workers.times do |num|
  God.watch do |w|

    w.dir        = rails_root
    w.name    = "resque-worker-#{num}"
    w.group   = 'rulingcom'

    w.interval = 30.seconds

    queue = case num
        when 0
                'critical'
        when 1..2
                'high,low'
        when 3..num_workers
                'low'
    end

    w.env       = {"QUEUE"=>queue, "RAILS_ENV"=>rails_env}
    w.start     = "HOME=#{user_home} QUEUE=* RAILS_ENV=#{rails_env} rake -f #{rails_root}/Rakefile resque:work"

    w.uid = (rails_env == 'production' )? "root" : development_uid
    w.gid = (rails_env == 'production' )? "root" : development_gid
  
     w.log = (rails_env == 'production' )? "/var/log/#{w.group}/#{w.name}.log":"#{rails_root}/log/dev-#{w.name}.log"

    # restart if memory gets too high
    w.transition(:up, :restart) do |on|
      on.condition(:memory_usage) do |c|
        c.above = 350.megabytes
        c.times = 2
      end
    end

    # determine the state on startup
    w.transition(:init, { true => :up, false => :start }) do |on|
      on.condition(:process_running) do |c|
        c.running = true
      end
    end

    # determine when process has finished starting
    w.transition([:start, :restart], :up) do |on|
      on.condition(:process_running) do |c|
        c.running = true
        c.interval = 5.seconds
      end

      # failsafe
      on.condition(:tries) do |c|
        c.times = 5
        c.transition = :start
        c.interval = 5.seconds
      end
    end

    # start if process is not running
    w.transition(:up, :start) do |on|
      on.condition(:process_running) do |c|
        c.running = false
      end
    end
  end
end

